<!--- Copyright 2023 xwst <xwst@gmx.net> --->

<!---
This file is part of sinaps.

sinaps is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

sinaps is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with sinaps. If not, see <http://www.gnu.org/licenses/>.
--->

# sinaps - sinaps is not a password safe
## Introduction
### Overview
sinaps is a stateless password manager that derives passwords using a single
master pin and different contexts (site names, user names, ...).
In that sense, it is very much like [lesspass](https://www.lesspass.com/) but it
also supports configuration files, encrypted secrets, a history and special
commands to set default values.

### Goals
sinaps aims to bridge the gap between classical and stateless password managers.
On the one hand, passwords should be deterministically generated on the fly to
avoid copying/syncing password databases all the time; on the other hand, it
should still be possible to also store small secrets that can not be chosen at
will (e.g. credit card numbers).
The following table provides an overview over all functions that shall be
supported at some point:

| **Feature**                    | **Description**                                                      | **Implemented?** |
|--------------------------------|----------------------------------------------------------------------|:----------------:|
| Stateless password derivation  | to avoid copying password databases and allow simple off site usage. | ✓                |
| Readable configuration file    | allowing to be printed out or (properly) version controlled.         | ✓                |
| Support for multiple OS        | as not everyone uses GNU/Linux, apparently.                          | ✗                |
| History for interactive inputs | to avoid retyping for passwords used multiple times in one session.  | ✓                |
| Support for storing secrets    | for passwords that can not be chosen at will.                        | ✗                |
| Include other config files     | to allow hierarchical "access rights" and shared config files.       | ✗                |

## License
### sinaps
sinaps is free software, licensed under the terms of the GNU General Public
License (GPLv3) or any later version.

If you want to learn more about free software, please consult the webpages of
the [GNU project](https://www.gnu.org/) or the
[free software foundation](https://fsf.org/). You may want to read the following
pages as a starting point:

- [What is free software? (GNU)](https://www.gnu.org/philosophy/free-sw.html.en)
- [Philosophy of the GNU project](https://www.gnu.org/philosophy/philosophy.html.en)
- [What is free software? (FSF)](https://www.fsf.org/about/what-is-free-software)

### Dependencies
#### Software
sinaps has a couple of dependencies that have their own licenses which, however,
are all [compatible](https://www.gnu.org/licenses/license-list) with the GPLv3.

- [libsodium](https://libsodium.org)
is used for hashing and all cryptographic computations in sinaps. It is licensed
under the ISC License.

- [gtkmm](https://www.gtkmm.org/)
is licensed under the terms of the LGPL and is used for frontend building.

- [libsigc++](https://libsigcplusplus.github.io/libsigcplusplus/)
is a LGPL-licensed call back framework used in gtkmm and also utilized by
sinaps.

- The [GNU readline library](https://tiswww.case.edu/php/chet/readline/rltop.html)
is used for reading user input in the command line interface of sinaps. It is
published under the terms of the GPLv3, too.

- Unit tests are created using the [CxxTest](https://github.com/CxxTest/cxxtest)
testing framework, which is available under the LGPLv3.

#### Tools
In addition, the following tools are used for convenience:

- [CMake](https://cmake.org/)
is the build automation tool used by sinaps and licensed under the terms of the
BSD 3-clause License.
- The sinaps source code is documented in the style of
[Doxygen](https://www.doxygen.nl/) comments. Doxygen can be used to build source
code documentations in various formats. It is licensed under the terms of the
GPLv2.

## Usage
### Startup
When starting sinaps without any command line arguments, it will work as a
completely stateless password manager. It will first ask for three pieces of
information, that are used exactly once to compute the internal main secret. The
two required numbers are Argon2 parameters, the used memory and the number of
iterations; they do not have to be confidential.
The third piece of information is your main password. It is used to derive all
passwords later on so it has to be reasonably strong and should be confidential.

These three inputs form one unit. Once you make a change (even a tiny one) to
any of them, you won't be able to retrieve the same passwords as before. So make
sure to remember all of them.

The particular choice for the Argon2 parameters will likely depend on your
setup. Please search the web for guidance on how to pick appropriate parameters,
as the suggestions are subject to change over time. Keep in mind, that your main
password should be strong, either way.

### Interactive Shell
#### Input Parsing
After entering the Argon2 parameters and your main password, you are presented
the interactive shell and can start deriving passwords. Every line of input is
hashed using your main password and encoded into a readable output but a certain
form of input is expected:
```
> context length counter alphabet skip
```

1. The context can be any string without whitespaces and is the input for the
   hash function. If you plan to use a configuration file, you may want to use
   a hierarchy like in unix paths. For example `http/website1/username`,
   `http/website2/other_username`, etc..
   This is the only mandatory input, as default values for the remaining input
   fields exist.
2. The length determines the length of the output; it has to be a positve
   integer. Changing the length does not alter the hashing itself so when you
   enter `my_context 10` and `my_context 12`, the first ten characters will be
   identical.
3. The counter can be increased to generate a new password for a given context
   without changing the context string itself.
4. The alphabet can be changed to specify the characters that may appear in the
   final password. Whitespaces are not allowed, but can be included by means of
   special substitutions. Enter the command `alphabet` to display a list of all
   automatic replacements in the alphabet string.

   Using `alphabet=words` is a special case which will generate a diceware-like
   password. In this case, the `lengt`-parameter specifies the number of words
   instead of characters.
5. This parameter allows to skip the first characters in the derived password,
   so that the last 4 characters of `my_context length=10` will be identical to
   the result of `my_context length=4 skip=6`.

The parameters are expected in the order given above, but you can also use
direct assignments like
```
> my_context alphabet=num length=4 1
```
After a direct assignment, the next parameter in the standard order is expected,
so the above example is identical to
```
> my_context alphabet=num length=4 counter=1
```

#### Line Editing
The shell uses the GNU readline library which supports a history, searching the
history and special line editing capabilities. Please consult the readline
documentation for details.


### Configuration Files
To do


## Technical Documentation
Technical details can be found in the "Namespaces"-section of the doxygen
documentation.
