// Copyright 2023 xwst <xwst@gmx.net>

/*
This file is part of sinaps.

sinaps is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

sinaps is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with sinaps. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef SINAPS_BACKEND_COLLECTION_H_
#define SINAPS_BACKEND_COLLECTION_H_

#include <list>
#include <memory>

#include <glibmm.h>

#include "contextnode.h"

namespace sinaps::Config {

class Collection : public ContextNode {

  public:

    Collection(
      gunichar,
      std::optional<Glib::ustring>,
      std::optional<uint64_t>,
      std::optional<uint16_t>,
      std::optional<uint32_t>
    );

    Glib::ustring get_path() override;
    Glib::ustring get_alphabet() override;
    uint64_t get_counter() override;
    uint16_t get_length() override;
    uint32_t get_skip() override;
    gunichar get_path_delimiter() override;
    Glib::ustring config_string();

    std::list<Glib::ustring> find_children(Glib::ustring, bool);
    ContextNodeParameters get_child_params(const Glib::ustring&);

  protected:
    gunichar delimiter;

    Glib::ustring config_string(Glib::ustring, uint = 0) override;
};
}

#endif // SINAPS_BACKEND_COLLECTION_H_
