// Copyright 2023 xwst <xwst@gmx.net>

/*
This file is part of sinaps.

sinaps is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

sinaps is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with sinaps. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef SINAPS_BACKEND_CONTEXTNODE_H_
#define SINAPS_BACKEND_CONTEXTNODE_H_

#include <list>
#include <memory>

#include <glibmm.h>

namespace sinaps::Config {


struct ContextNodeParameters {
  Glib::ustring name;
  std::optional<Glib::ustring> alphabet;
  std::optional<uint64_t> counter;
  std::optional<uint16_t> length;
  std::optional<uint32_t> skip;
};


class ContextNode {

  private:
    ContextNode* parent;

  protected:
    std::list<std::shared_ptr<ContextNode>> children;
    ContextNodeParameters params;

    struct ctor_access_token {
      ContextNode* parent;
      explicit ctor_access_token(ContextNode* parent) : parent(parent) {};
    };

    ContextNode* get_child(const Glib::ustring& child_name);
    ContextNode* get_child_recursively(const Glib::ustring& path);
    Glib::ustring optionals_to_string();
    void find_children(std::list<Glib::ustring>&, Glib::ustring, bool);

  public:
    ContextNode() = delete;
    ContextNode(const ContextNode&) = delete;
    ContextNode& operator=(const ContextNode&) = delete;
    ContextNode(const ContextNode&&) = delete;
    ContextNode& operator=(const ContextNode&&) = delete;
    virtual ~ContextNode() = default;

    ContextNode(ctor_access_token c, const ContextNodeParameters& params);

    ContextNodeParameters get_parameters();
    Glib::ustring get_name() { return this->params.name; }
    virtual Glib::ustring get_path();
    virtual Glib::ustring get_alphabet();
    virtual uint64_t get_counter();
    virtual uint16_t get_length();
    virtual uint32_t get_skip();
    virtual gunichar get_path_delimiter();

    bool has_children();
    bool contains(const Glib::ustring&);
    std::shared_ptr<ContextNode> add_child(const ContextNodeParameters&);
    virtual Glib::ustring config_string(Glib::ustring, uint = 0);
    virtual Glib::ustring to_string(bool = false);

};
}

#endif // SINAPS_BACKEND_CONTEXTNODE_H_
