// Copyright 2023 xwst <xwst@gmx.net>

/*
This file is part of sinaps.

sinaps is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

sinaps is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with sinaps. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef SINAPS_BACKEND_PARSER_H_
#define SINAPS_BACKEND_PARSER_H_

#include <memory>

#include "collection.h"
#include "contextnode.h"

namespace sinaps::Config {


template <typename T>
bool parse_input_data(const Glib::ustring& input, T& dest) {
  std::istringstream stream(input);
  stream >> dest;
  return stream.eof() and not stream.fail();
}


class ConfigParser {

  private:
    template <typename T> class StackMap;

    static gunichar delimiter;
    static gunichar comment;
    std::unique_ptr<StackMap<std::shared_ptr<ContextNode>>> levels;
    std::shared_ptr<Collection> result;

    ConfigParser();
    // void add_child_recursively(
    //   Glib::ustring name,
    //   std::optional<Glib::ustring> alphabet,
    //   std::optional<uint64_t> counter,
    //   std::optional<uint16_t> length,
    //   std::optional<uint32_t> skip
    // );
    void remove_comments(Glib::ustring&);
    void parse_line(Glib::ustring&);
    std::shared_ptr<Collection> parse(std::string);

    static uint ltrim(Glib::ustring&);
    template<typename T>
    static T str2int(const Glib::ustring&, T, const std::string&);

  public:
    static std::shared_ptr<Collection> parse_file(std::string);
    static ContextNodeParameters parse_context(std::vector<Glib::ustring>&);
};

}

#endif // SINAPS_BACKEND_PARSER_H_
