// Copyright 2023 xwst <xwst@gmx.net>

/*
This file is part of sinaps.

sinaps is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

sinaps is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with sinaps. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef SINAPS_BACKEND_UTIL_H_
#define SINAPS_BACKEND_UTIL_H_

#include <vector>

#include <glibmm.h>

namespace sinaps {


/**
  * @brief The default value for ContextNode::alphabet.
  */
inline const Glib::ustring default_alphabet
  = "alphanum";



/**
  * @brief A map storing short names and corresponding character sets that are
  *        meant to replace the short names prior to password derivation.
  *
  * This map is not const because it may me modified at runtime.
  */
  static std::map<Glib::ustring, Glib::ustring> alphabet_abbreviations{
  {"a-z", "abcdefghijklmnopqrstuvwxyz"},
  {"A-Z", "ABCDEFGHIJKLMNOPQRSTUVWXYZ"},
  {"a-Z", "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"},
  {"0-9", "0123456789"},
  {"alphanum",
    "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"},
  {"alpha", "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"},
  {"num", "0123456789"},
  {"whitespace", " "},
  {"hex", "0123456789abcdef"},
  {"HEX", "0123456789ABCDEF"},
  {"special", "!\"§$%&/()=?{}[]\\+*~#'-_.:,;<>| @"}
};


/**
  * @brief The default value for ContextNode::counter.
  */
inline constexpr uint64_t default_counter = 0;


/**
  * @brief The default value for ContextNode::length.
  */
inline constexpr uint16_t default_length = 25;


/**
  * @brief The default value for ContextNode::skip.
  */
inline constexpr uint32_t default_skip = 0;


/**
  * \brief Converts a ustring (UTF-8) to a vector of UTF-32 characters
  *
  * The underlying cryptographic functions use std::vector objects for
  * which UTF-8 characters (variable width) are not suitable.
  */
inline std::vector<gunichar> ustr_to_UCS4_vec(const Glib::ustring& s) {
    std::vector<gunichar> result;
    result.reserve(s.size());

    for(gunichar gc : s) result.push_back(gc);

    return result;
}

}

#endif
