// Copyright 2023 xwst <xwst@gmx.net>

/*
This file is part of sinaps.

sinaps is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

sinaps is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with sinaps. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef SINAPS_FRONTEND_READLINE_APPLICATION_H
#define SINAPS_FRONTEND_READLINE_APPLICATION_H

#include <giomm.h>

#include "frontend/frontend_interface.h"
#include "frontend/readline_proxy.h"

#include <iostream>

namespace sinaps {


class ReadlineApplication : public FrontendInterface, public Gio::Application {

  protected:
    ReadlineProxy& rp;
    bool keep_running;

    std::shared_ptr<char> get_line(const char*, bool);
    FrontendInterface::GCV_uptr get_pin(int);
    template <typename T>
    bool get_numeric_value(T&, const char*, T);

  public:
    ReadlineApplication();

    /*****************************************************/
    /********************* Overrides *********************/
    /*****************************************************/
    void set_input_completions(std::list<Glib::ustring>) override;
    void display_password(GCV_uptr&) override;
    void password_to_clipboard(GCV_uptr&) override;
    void clear_clipboard() override;
    void clear_output() override;
    void quit() override;
    void start() override;
    void display_info(const Glib::ustring&) override;
    void accept_user_input() override;
    void prompt_argon2_parameters(
      const Glib::ustring&,
      unsigned long long,
      size_t
    ) override;

    void on_activate() override;
};

}

#endif // SINAPS_FRONTEND_READLINE_APPLICATION_H
