// Copyright 2023 xwst <xwst@gmx.net>

/*
This file is part of sinaps.

sinaps is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

sinaps is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with sinaps. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef SINAPS_READLINE_PROXY_H_
#define SINAPS_READLINE_PROXY_H_

#include <list>
#include <memory>

#include <glibmm.h>
#include <readline/readline.h>
#include <sigc++/sigc++.h>

namespace sinaps {

class ReadlineProxy {
  private:
    Glib::ustring current_line_buffer;
    std::list<Glib::ustring> readline_completions;

    bool has_input_buffer_changed();
    static void redisplay_hook();
    static char* completion_entry_func(const char*, int);
    static char** tab_completion(const char*, int, int);

  protected:
    ReadlineProxy();
    ReadlineProxy(const ReadlineProxy&) = delete;
    ReadlineProxy& operator=(const ReadlineProxy&) = delete;
    ~ReadlineProxy() { };

  public:

    static ReadlineProxy& get_instance();

    std::shared_ptr<char> get_line(const char*, bool);



    /*****************************************************/
    /*********************** Slots ***********************/
    /*****************************************************/
    void set_completions(std::list<Glib::ustring>);

    /*****************************************************/
    /********************** Signals **********************/
    /*****************************************************/
    sigc::signal<void(const Glib::ustring&)> input_buffer_changed;
};

}

#endif // SINAPS_READLINE_PROXY_H_
