// Copyright 2023 xwst <xwst@gmx.net>

/*
This file is part of sinaps.

sinaps is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

sinaps is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with sinaps. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef SINAPS_SECURE_MEMORY_UTIL_H_
#define SINAPS_SECURE_MEMORY_UTIL_H_

#include <vector>

#include <sodium.h>

#include "secure_memory/constants.h"
#include "secure_memory/vector.h"

namespace sinaps::SecureMemory::util {

    static constexpr size_t CHACHA20_BLOCKBYTES = 64;
    static constexpr size_t PW_SEED_LENGTH
                            = crypto_stream_chacha20_ietf_KEYBYTES;
    static constexpr size_t PW_BLOCK_SIZE = CHACHA20_BLOCKBYTES;

    static constexpr unsigned char chacha_block_zeros [PW_BLOCK_SIZE] = {0};

    SecureMemory::UCV_uptr asciiBitString(const SecureMemory::UCV&);
    std::vector<unsigned char> binaryRepresentation(uint64_t, size_t);
    std::vector<unsigned char> counter2nonce(uint64_t);
    unsigned int block_modulo(const SecureMemory::UCV&, unsigned int);

    void derivePasswordBlock(const SecureMemory::UCV&,
                             std::vector<unsigned char> const&, uint32_t,
                             SecureMemory::UCV&);

} // namespace sinaps::SecureMemory::util

#endif // SINAPS_SECURE_MEMORY_UTIL_H_
