// Copyright 2023 xwst <xwst@gmx.net>

/*
This file is part of sinaps.

sinaps is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

sinaps is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with sinaps. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef SINAPS_SECURE_MEMORY_CHARVECTOR_H_
#define SINAPS_SECURE_MEMORY_CHARVECTOR_H_

#include <memory>
#include <vector>

#include "secure_memory/sodiumallocator.h"

namespace sinaps::SecureMemory {

    template <typename T>
    using Vector = std::vector<T, SodiumAllocator<T>>;
    template <typename T>
    using Vector_uptr = std::unique_ptr<Vector<T>>;

    using UCV = Vector<unsigned char>;
    using UCV_uptr = Vector_uptr<unsigned char>;

    template <typename T>
    void setNoAccess(std::unique_ptr<Vector<T>>&);
    template <typename T>
    void setReadOnly(std::unique_ptr<Vector<T>>&);
    template <typename T>
    void setReadWrite(std::unique_ptr<Vector<T>>&);

} // namespace sinaps::SecureMemory

#endif // SINAPS_SECURE_MEMORY_CHARVECTOR_H_
