// Copyright 2023 xwst <xwst@gmx.net>

/*
This file is part of sinaps.

sinaps is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

sinaps is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with sinaps. If not, see <http://www.gnu.org/licenses/>.
*/

#include <sigc++/sigc++.h>

#include "backend/backend.h"
#include "frontend/readline_application.h"


int main(int argc, char* argv[]) {
  setlocale (LC_ALL, "");

  sinaps::Backend backend;
  auto frontend = sinaps::ReadlineApplication();

  #define CONNECT(sender, signal, receiver, slot) \
    sender.signal.connect(sigc::mem_fun(receiver, &sinaps::slot));
  #define F2B(signal, slot) CONNECT(frontend, signal, backend, Backend::slot)
  #define B2F(signal, slot) \
    CONNECT(backend, signal, frontend, FrontendInterface::slot)

  frontend.signal_command_line().connect(
    sigc::mem_fun(backend, &sinaps::Backend::parse_commandline_arguments),
    false
  );

  F2B(signal_ready, on_frontend_ready)
  F2B(signal_opslimit_entered, set_opslimit)
  F2B(signal_memlimit_entered, set_memlimit)
  F2B(signal_pin_entered, set_pin)
  F2B(signal_line_entered, on_line_entered)
  F2B(signal_input_buffer_changed, on_input_buffer_changed)

  B2F(signal_display_info, display_info)
  B2F(signal_start_frontend, start)
  B2F(signal_start_input_loop, accept_user_input)
  B2F(signal_argon2_parameters_missing, prompt_argon2_parameters)
  B2F(signal_exit, quit)
  B2F(signal_clear_output, clear_output)
  B2F(signal_print_info, print_info)
  B2F(signal_display_password, display_password)
  B2F(signal_new_input_completions, set_input_completions)

  frontend.hold();

  return frontend.run(argc, argv);
}
