// Copyright 2023 xwst <xwst@gmx.net>

/*
This file is part of sinaps.

sinaps is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

sinaps is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with sinaps. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef SINAPS_UNITTEST_HELPERS_H_
#define SINAPS_UNITTEST_HELPERS_H_

#include <cxxtest/TestSuite.h>

#include "secure_memory/vector.h"

namespace sinaps::UnitTests {

    class RandomParameters {

        public:
            SecureMemory::Vector_uptr<char> pin;
            std::vector<char> context;
            std::vector<char> alphabet;
            std::vector<char> encoding;
            uint64_t counter;
            int opslimit;
            int memlimit; // used memory in kibibytes (= bytes/1024)
            int length;

            RandomParameters();

        private:
            size_t random(size_t, size_t);
            template <typename V>
            void randomString(V&);
    };

    void randomPasswordDerivationTests(int);
    void passwordAsSequenceTests(int, int);

} // namespace sinaps::UnitTests

#endif // SINAPS_UNITTEST_HELPERS_H_
